@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 
 # This file is part of ReText
-# Copyright: 2012-2021 Dmitry Shachnev
+# Copyright: 2012-2022 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import sys
 import markups
 import markups.common
-from os.path import dirname, exists, join, expanduser
+from os.path import abspath, dirname, join, expanduser
 
-from PyQt5.QtCore import QByteArray, QLocale, QSettings, QStandardPaths
-from PyQt5.QtGui import QFont, QFontDatabase
+from PyQt6.QtCore import QByteArray, QLocale, QSettings
+from PyQt6.QtGui import QFont, QFontDatabase
 
-app_version = "7.2.0"
+app_version = "8.0.1"
 
 settings = QSettings('ReText project', 'ReText')
 
@@ -33,36 +32,12 @@ if not str(settings.fileName()).endswith('.conf'):
 	settings = QSettings(QSettings.Format.IniFormat, QSettings.Scope.UserScope,
 		'ReText project', 'ReText')
 
-datadirs = []
 
-def initializeDataDirs():
-	assert not datadirs
-
-	try:
-		datadirs.append(dirname(dirname(__file__)))
-	except NameError:
-		pass
-
-	dataLocations = QStandardPaths.standardLocations(QStandardPaths.StandardLocation.GenericDataLocation)
-	datadirs.extend(join(d, 'retext') for d in dataLocations)
-
-	if sys.platform == "win32":
-		# Windows compatibility: Add "PythonXXX\share\" path
-		datadirs.append(join(dirname(sys.executable), 'share', 'retext'))
-
-	# For virtualenvs
-	datadirs.append(join(dirname(dirname(sys.executable)), 'share', 'retext'))
-
-_iconPath = None
+packageDir = abspath(dirname(__file__))
 
 def getBundledIcon(iconName):
-	global _iconPath
-	if _iconPath is None:
-		for dir in ['icons'] + datadirs:
-			_iconPath = join(dir, 'icons')
-			if exists(_iconPath):
-				break
-	return join(_iconPath, iconName + '.png')
+	return join(packageDir, 'icons', iconName + '.png')
+
 
 configOptions = {
 	'appStyleSheet': '',
@@ -73,8 +48,8 @@ configOptions = {
 	'detectEncoding': True,
 	'directoryPath': expanduser("~"),
 	'documentStatsEnabled': False,
-	'editorFont': QFont(),
-	'font': QFont(),
+	'editorFont': '',
+	'font': '',
 	'handleWebLinks': False,
 	'hideToolBar': False,
 	'highlightCurrentLine': 'disabled',
@@ -104,17 +79,12 @@ configOptions = {
 	'uiLanguage': QLocale.system().name(),
 	'useFakeVim': False,
 	'useWebEngine': False,
-	'useWebKit': False,
 	'wideCursor': False,
 	'windowGeometry': QByteArray(),
 	'windowTitleFullPath': False,
 }
 
 def readFromSettings(key, keytype, settings=settings, default=None):
-	if isinstance(default, QFont):
-		family = readFromSettings(key, str, settings, default.family())
-		size = readFromSettings(key + 'Size', int, settings, 0)
-		return QFont(family, size)
 	if not settings.contains(key):
 		return default
 	try:
@@ -138,10 +108,7 @@ def readListFromSettings(key, settings=settings):
 		return value
 
 def writeToSettings(key, value, default, settings=settings):
-	if isinstance(value, QFont):
-		writeToSettings(key, value.family(), '', settings)
-		writeToSettings(key + 'Size', max(value.pointSize(), 0), 0, settings)
-	elif value == default:
+	if value == default:
 		settings.remove(key)
 	else:
 		settings.setValue(key, value)
@@ -158,7 +125,7 @@ def getSettingsFilePath(settings=settings):
 	return settings.fileName()
 
 
-class ReTextSettings(object):
+class ReTextSettings:
 	def __init__(self):
 		for option in configOptions:
 			value = configOptions[option]
@@ -171,15 +138,17 @@ class ReTextSettings(object):
 		object.__setattr__(self, option, value)
 		writeToSettings(option, value, configOptions[option])
 
-	def __getattribute__(self, option):
-		value = object.__getattribute__(self, option)
-		# Choose a font just-in-time, because when the settings are
-		# loaded it is too early to work.
-		if option == 'font' and not value.family():
-			value = QFont()
-		if option == 'editorFont' and not value.family():
-			value = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
-		return value
+	def getPreviewFont(self):
+		font = QFont()
+		if self.font:
+			font.fromString(self.font)
+		return font
+
+	def getEditorFont(self):
+		font = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
+		if self.editorFont:
+			font.fromString(self.editorFont)
+		return font
 
 globalSettings = ReTextSettings()
 
