@@ -2,7 +2,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 
 # ReText
-# Copyright 2011-2021 Dmitry Shachnev
+# Copyright 2011-2023 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,15 +24,14 @@ import signal
 import markups
 from os import devnull
 from os.path import join
-from ReText import datadirs, settings, globalSettings, app_version
-from ReText import initializeDataDirs
+from ReText import packageDir, settings, globalSettings, app_version
 from ReText.window import ReTextWindow
 
-from PyQt5.QtCore import QCommandLineOption, QCommandLineParser, QFile, \
+from PyQt6.QtCore import QCommandLineOption, QCommandLineParser, QFile, \
  QFileInfo, QIODevice, QLibraryInfo, QTextStream, QTranslator, Qt
-from PyQt5.QtWidgets import QApplication
-from PyQt5.QtNetwork import QNetworkProxyFactory
-from PyQt5.QtDBus import QDBusConnection, QDBusInterface
+from PyQt6.QtWidgets import QApplication
+from PyQt6.QtNetwork import QNetworkProxyFactory
+from PyQt6.QtDBus import QDBusConnection, QDBusInterface
 
 def canonicalize(option):
 	if option == '-':
@@ -60,7 +59,6 @@ def main():
 
 	# Needed for Qt WebEngine on Windows
 	QApplication.setAttribute(Qt.ApplicationAttribute.AA_ShareOpenGLContexts)
-	QApplication.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps)
 	app = QApplication(sys.argv)
 	app.setOrganizationName("ReText project")
 	app.setApplicationName("ReText")
@@ -70,15 +68,12 @@ def main():
 	app.setDesktopFileName('me.mitya57.ReText.desktop')
 	QNetworkProxyFactory.setUseSystemConfiguration(True)
 
-	initializeDataDirs()
 	RtTranslator = QTranslator()
-	for path in datadirs:
-		if RtTranslator.load('retext_' + globalSettings.uiLanguage,
-		                     join(path, 'locale')):
-			break
+	RtTranslator.load('retext_' + globalSettings.uiLanguage,
+	                  join(packageDir, 'locale'))
 	QtTranslator = QTranslator()
-	QtTranslator.load("qtbase_" + globalSettings.uiLanguage,
-		QLibraryInfo.location(QLibraryInfo.LibraryLocation.TranslationsPath))
+	translationsPath = QLibraryInfo.path(QLibraryInfo.LibraryPath.TranslationsPath)
+	QtTranslator.load("qtbase_" + globalSettings.uiLanguage, translationsPath)
 	app.installTranslator(RtTranslator)
 	app.installTranslator(QtTranslator)
 
@@ -106,25 +101,26 @@ def main():
 		sheetfile.close()
 	window = ReTextWindow()
 
+	# ReText can change directory when loading files, so we
+	# need to have a list of canonical names before loading
+	fileNames = list(map(canonicalize, filesToOpen))
+
 	openInExistingWindow = (globalSettings.openFilesInExistingWindow
 		and not parser.isSet(newWindowOption))
 	connection = QDBusConnection.sessionBus()
 	if connection.isConnected() and openInExistingWindow:
 		connection.registerObject('/', window, QDBusConnection.RegisterOption.ExportAllSlots)
 		serviceName = 'me.mitya57.ReText'
-		if not connection.registerService(serviceName) and filesToOpen:
+		if not connection.registerService(serviceName) and fileNames:
 			print('Opening the file(s) in the existing window of ReText.')
 			iface = QDBusInterface(serviceName, '/', '', connection)
-			for fileName in filesToOpen:
+			for fileName in fileNames:
 				iface.call('openFileWrapper', fileName)
 			qWidgetIface = QDBusInterface(serviceName, '/', 'org.qtproject.Qt.QWidget', connection)
 			qWidgetIface.call('raise')
 			sys.exit(0)
 
 	window.show()
-	# ReText can change directory when loading files, so we
-	# need to have a list of canonical names before loading
-	fileNames = list(map(canonicalize, filesToOpen))
 	readStdIn = False
 
 	if globalSettings.openLastFilesOnStartup:
