@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 #
 # This file is part of ReText
-# Copyright: 2012-2021 Dmitry Shachnev
+# Copyright: 2012-2023 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@ import re
 import weakref
 
 from markups import MarkdownMarkup, ReStructuredTextMarkup, TextileMarkup
-from ReText import globalSettings, tablemode, readFromSettings
+from ReText import globalSettings, settings, tablemode
 
-from PyQt5.QtCore import pyqtSignal, QFileInfo, QPoint, QRect, QSize, Qt
-from PyQt5.QtGui import QColor, QImage, QKeyEvent, QMouseEvent, QPainter, \
+from PyQt6.QtCore import pyqtSignal, QFileInfo, QPoint, QRect, QSize, Qt
+from PyQt6.QtGui import QAction, QColor, QImage, QKeyEvent, QMouseEvent, QPainter, \
 QPalette, QTextCursor, QTextFormat, QWheelEvent, QGuiApplication
-from PyQt5.QtWidgets import QAction, QApplication, QFileDialog, QLabel, QTextEdit, QWidget
+from PyQt6.QtWidgets import QApplication, QFileDialog, QLabel, QTextEdit, QWidget
 
 try:
 	from ReText.fakevimeditor import ReTextFakeVimHandler
@@ -34,19 +34,44 @@ except ImportError:
 	ReTextFakeVimHandler = None
 
 colors = {
-	'marginLine': QColor(0xdc, 0xd2, 0xdc),
-	'currentLineHighlight': QColor(0xff, 0xff, 0xc8),
-	'infoArea': QColor(0xaa, 0xff, 0x55, 0xaa),
-	'statsArea': QColor(0xff, 0xaa, 0x55, 0xaa),
-	'lineNumberArea': Qt.GlobalColor.cyan,
-	'lineNumberAreaText': Qt.GlobalColor.darkCyan
+	# Editor
+	'marginLine':           {'light': '#dcd2dc', 'dark': '#3daee9'},
+	'currentLineHighlight': {'light': '#ffffc8', 'dark': '#31363b'},
+	'infoArea':             {'light': '#aaaaff55', 'dark': '#aa557f2a'},
+	'statsArea':            {'light': '#aaffaa55', 'dark': '#aa7f552a'},
+	'lineNumberArea':       {'light': '#00ffff', 'dark': '#31363b'},
+	'lineNumberAreaText':   {'light': '#008080', 'dark': '#bdc3c7'},
+	# Highlighter
+	'htmlTags':             {'light': '#800080', 'dark': '#d070d0'},
+	'htmlSymbols':          {'light': '#008080', 'dark': '#70d0a0'},
+	'htmlStrings':          {'light': '#808000', 'dark': '#d0d070'},
+	'htmlComments':         {'light': '#a0a0a4', 'dark': '#b0b0aa'},
+	'codeSpans':            {'light': '#505050', 'dark': '#afafaf'},
+	'markdownLinks':        {'light': '#000090', 'dark': '#8080ff'},
+	'blockquotes':          {'light': '#808080', 'dark': '#b0b0b0'},
+	'restDirectives':       {'light': '#800080', 'dark': '#d070d0'},
+	'restRoles':            {'light': '#800000', 'dark': '#d07070'},
+	'whitespaceOnEnd':      {'light': '#80e1e1a5', 'dark': '#8096966e'},
 }
 
-colorValues = {
-	colorName: readFromSettings(
-		'ColorScheme/' + colorName, QColor, default=colors[colorName])
-	for colorName in colors
-}
+colorValues = {}
+
+def updateColorScheme(settings=settings):
+	palette = QApplication.palette()
+	windowColor = palette.color(QPalette.ColorRole.Window)
+	themeVariant = 'light' if windowColor.lightness() > 150 else 'dark'
+	settings.beginGroup('ColorScheme')
+	for key in colors:
+		if settings.contains(key):
+			colorValues[key] = settings.value(key, type=QColor)
+		else:
+			colorValues[key] = QColor(colors[key][themeVariant])
+	settings.endGroup()
+
+def getColor(colorName):
+	if not colorValues:
+		updateColorScheme()
+	return colorValues[colorName]
 
 def documentIndentMore(document, cursor, globalSettings=globalSettings):
 	if cursor.hasSelection():
@@ -101,6 +126,7 @@ class ReTextEdit(QTextEdit):
 		Qt.Key.Key_Underscore,
 		Qt.Key.Key_Asterisk,
 		Qt.Key.Key_QuoteDbl,
+		Qt.Key.Key_QuoteLeft,
 		Qt.Key.Key_Apostrophe
 	]
 	surroundKeysOtherClose = {
@@ -136,7 +162,7 @@ class ReTextEdit(QTextEdit):
 			self.setLineWrapMode(QTextEdit.LineWrapMode.WidgetWidth)
 
 	def updateFont(self):
-		self.setFont(globalSettings.editorFont)
+		self.setFont(globalSettings.getEditorFont())
 		metrics = self.fontMetrics()
 		self.marginx = (int(self.document().documentMargin())
 			+ metrics.horizontalAdvance(' ' * globalSettings.rightMargin))
@@ -152,7 +178,7 @@ class ReTextEdit(QTextEdit):
 		if not globalSettings.rightMargin:
 			return super().paintEvent(event)
 		painter = QPainter(self.viewport())
-		painter.setPen(colorValues['marginLine'])
+		painter.setPen(getColor('marginLine'))
 		y1 = self.rect().topLeft().y()
 		y2 = self.rect().bottomLeft().y()
 		painter.drawLine(self.marginx, y1, self.marginx, y2)
@@ -161,7 +187,7 @@ class ReTextEdit(QTextEdit):
 	def wheelEvent(self, event):
 		modifiers = QGuiApplication.keyboardModifiers()
 		if modifiers == Qt.KeyboardModifier.ControlModifier:
-			font = globalSettings.editorFont
+			font = globalSettings.getEditorFont()
 			size = font.pointSize()
 			scroll = event.angleDelta().y()
 			if scroll > 0:
@@ -206,15 +232,18 @@ class ReTextEdit(QTextEdit):
 		cursor = self.cursorForPosition(event.pos())
 		curchar = self.document().characterAt(cursor.position())
 		isalpha = curchar.isalpha()
-		dictionary = self.tab.highlighter.dictionary
 		word = None
 		if isalpha and not (oldcursor.hasSelection() and oldcursor.selectedText() != cursor.selectedText()):
 			cursor.select(QTextCursor.SelectionType.WordUnderCursor)
 			word = cursor.selectedText()
 
-		if word is not None and dictionary and not dictionary.check(word):
+		correct = True
+		if word is not None and self.tab.highlighter.dictionaries:
+			correct = any(dictionary.check(word) for dictionary in self.tab.highlighter.dictionaries)
+
+		if not correct:
 			self.setTextCursor(cursor)
-			suggestions = dictionary.suggest(word)
+			suggestions = self.tab.highlighter.dictionaries[0].suggest(word)
 			actions = [self.parent.act(sug, trig=self.fixWord(sug)) for sug in suggestions]
 			menu.insertSeparator(menu.actions()[0])
 			for action in actions[::-1]:
@@ -239,11 +268,10 @@ class ReTextEdit(QTextEdit):
 		block = cursor.block()
 		cursor.clearSelection()
 		self.setTextCursor(cursor)
-		dictionary = self.tab.highlighter.dictionary
-		if (dictionary is None) or not newword:
-			return
-		dictionary.add(newword)
-		self.tab.highlighter.rehighlightBlock(block)
+		if self.tab.highlighter.dictionaries and newword:
+			dictionary = self.tab.highlighter.dictionaries[0]
+			dictionary.add(newword)
+			self.tab.highlighter.rehighlightBlock(block)
 
 	def isSurroundKey(self, key):
 		return key in self.surroundKeysSelfClose or key in self.surroundKeysOtherClose
@@ -375,7 +403,7 @@ class ReTextEdit(QTextEdit):
 		if globalSettings.highlightCurrentLine == 'disabled':
 			return self.setExtraSelections([])
 		selection = QTextEdit.ExtraSelection()
-		selection.format.setBackground(colorValues['currentLineHighlight'])
+		selection.format.setBackground(getColor('currentLineHighlight'))
 		selection.format.setProperty(QTextFormat.Property.FullWidthSelection, True)
 		selection.cursor = self.textCursor()
 		selection.cursor.clearSelection()
@@ -384,7 +412,7 @@ class ReTextEdit(QTextEdit):
 			selections.append(QTextEdit.ExtraSelection())
 			selections[0].cursor.movePosition(QTextCursor.MoveOperation.StartOfBlock)
 			selections[0].cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock, QTextCursor.MoveMode.KeepAnchor)
-			selections[1].format.setBackground(colorValues['currentLineHighlight'])
+			selections[1].format.setBackground(getColor('currentLineHighlight'))
 			selections[1].format.setProperty(QTextFormat.Property.FullWidthSelection, True)
 			selections[1].cursor = self.textCursor()
 			selections[1].cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock)
@@ -493,6 +521,11 @@ class ReTextEdit(QTextEdit):
 		characterCount = len(text)
 		self.statistics = (wordCount, alphaNumCount, characterCount)
 
+	def inputMethodEvent(self, event):
+		# Empty events cause layout issues (issue #597), so ignore them.
+		if event.preeditString() or event.commitString() or event.attributes():
+			super().inputMethodEvent(event)
+
 
 class LineNumberArea(QWidget):
 	def __init__(self, editor):
@@ -506,8 +539,8 @@ class LineNumberArea(QWidget):
 		if not globalSettings.lineNumbersEnabled:
 			return super().paintEvent(event)
 		painter = QPainter(self)
-		painter.fillRect(event.rect(), colorValues['lineNumberArea'])
-		painter.setPen(colorValues['lineNumberAreaText'])
+		painter.fillRect(event.rect(), getColor('lineNumberArea'))
+		painter.setPen(getColor('lineNumberAreaText'))
 		cursor = self.editor.cursorForPosition(QPoint(0, 0))
 		atEnd = False
 		fontHeight = self.fontMetrics().height()
@@ -593,7 +626,7 @@ class InfoArea(QLabel):
 
 class LineInfoArea(InfoArea):
 	def __init__(self, editor):
-		InfoArea.__init__(self, editor, colorValues['infoArea'])
+		InfoArea.__init__(self, editor, getColor('infoArea'))
 
 	def getAreaPosition(self, width, height):
 		viewport = self.editor.viewport()
@@ -613,7 +646,7 @@ class LineInfoArea(InfoArea):
 
 class TextInfoArea(InfoArea):
 	def __init__(self, editor):
-		InfoArea.__init__(self, editor, colorValues['statsArea'])
+		InfoArea.__init__(self, editor, getColor('statsArea'))
 
 	def getAreaPosition(self, width, height):
 		viewport = self.editor.viewport()
